/*
 * $Id: brctl.c,v 1.8 2000/01/19 10:38:49 root Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libbridge.h"

int nametoindex(char *name)
{
	int ifindex;

	if ((ifindex = if_nametoindex(name)) != 0)
		return ifindex;

	fprintf(stderr, "interface %s doesn't exist\n", name);
	exit(-1);
}



void br_cmd_addbr(struct bridge *br, char *arg0, char *arg1)
{
	br_add_bridge(arg0);
}

void br_cmd_delbr(struct bridge *br, char *arg0, char *arg1)
{
	br_del_bridge(arg0);
}

void br_cmd_addif(struct bridge *br, char *arg0, char *arg1)
{
	br_add_interface(br, nametoindex(arg0));
}

void br_cmd_delif(struct bridge *br, char *arg0, char *arg1)
{
	br_del_interface(br, nametoindex(arg0));
}

void br_cmd_show(struct bridge *br, char *arg0, char *arg1)
{
	if (arg0 != NULL && (br = br_find_bridge(arg0)) != NULL) {
		br_dump_info(br);
		return;
	}

	printf("bridge name\tbridge id\t\tSTP enabled\n");
	br = bridge_list;
	while (br != NULL) {
		printf("%s\t\t", br->ifname);
		br_dump_bridge_id((unsigned char *)&br->info.bridge_id);
		printf("\t%s\n", br->info.stp_enabled?"yes":"no");

		br = br->next;
	}
}

static int compare_fdbs(const void *_f0, const void *_f1)
{
	const struct fdb_entry *f0 = _f0;
	const struct fdb_entry *f1 = _f1;

	if (f0->port_no < f1->port_no)
		return -1;

	if (f0->port_no > f1->port_no)
		return 1;

	return memcmp(f0->mac_addr, f1->mac_addr, 6);
}

void br_cmd_showmacs(struct bridge *br, char *arg0, char *arg1)
{
	struct fdb_entry fdb[1024];
	int i;
	int num;

	num = br_get_fdb_list(br, fdb, 1024);
	qsort(fdb, num, sizeof(struct fdb_entry), compare_fdbs);

	printf("port\tmac addr\t\tis local?\tageing timer\n");
	for (i=0;i<num;i++) {
		struct fdb_entry *f;

		f = fdb + i;
		printf("%3i\t", f->port_no);
		printf("%.2x:%.2x:%.2x:%.2x:%.2x:%.2x\t",
		       f->mac_addr[0], f->mac_addr[1], f->mac_addr[2],
		       f->mac_addr[3], f->mac_addr[4], f->mac_addr[5]);
		printf("%s\t\t", f->is_local?"yes":"no");
		br_show_timer(f->ageing_timer_value);
		printf("\n");
	}
}








struct command
{
	int needs_bridge_argument;
	char *name;
	void (*func)(struct bridge *br, char *arg0, char *arg1);
} commands[] = {
	{0, "addbr", br_cmd_addbr},
	{1, "addif", br_cmd_addif},
	{0, "delbr", br_cmd_delbr},
	{1, "delif", br_cmd_delif},
	{0, "show", br_cmd_show},
	{1, "showmacs", br_cmd_showmacs},
};

struct command *lookup(char *cmd)
{
	int i;
	int numcommands;

	numcommands = sizeof(commands)/sizeof(commands[0]);

	for (i=0;i<numcommands;i++)
		if (!strcmp(cmd, commands[i].name))
			return &commands[i];

	return NULL;
}

void help()
{
	fprintf(stderr, "syntax:\n");
	fprintf(stderr, "\taddbr <bridge name>\t\t\tadd bridge\n");
	fprintf(stderr, "\taddif <bridge name> <interface name>\tadd interface to bridge\n");
	fprintf(stderr, "\tdelbr <bridge name>\t\t\tdelete bridge\n");
	fprintf(stderr, "\tdelif <bridge name> <interface name>\tdelete interface from bridge\n");
	fprintf(stderr, "\tshow\t\t\t\t\tshow a list of bridges\n");
	fprintf(stderr, "\tshow  <bridge name>\t\t\tshow bridge info\n");
	fprintf(stderr, "\tshowmacs\t\t\t\tshow a list of mac entries for this bridge\n");
}

int main(int argc, char *argv[])
{
	int argindex;
	struct bridge *br;
	struct command *cmd;

	br_init();

	if (argc < 2)
		goto help;

	if ((cmd = lookup(argv[1])) == NULL) {
		fprintf(stderr, "never heard of command [%s]\n", argv[1]);
		goto help;
	}

	argindex = 2;
	br = NULL;
	if (cmd->needs_bridge_argument) {
		if (argindex >= argc) {
			fprintf(stderr, "this option requires a bridge name as argument\n");
			return 1;
		}

		br = br_find_bridge(argv[argindex]);

		if (br == NULL) {
			fprintf(stderr, "bridge %s doesn't exist!\n", argv[argindex]);
			return 1;
		}

		argindex++;
	}

	cmd->func(br, argv[argindex], argv[argindex+1]);

	return 0;

help:
	help();
	return 1;
}
