/*
 * $Id: libbridge.h,v 1.3 2000/01/19 12:37:43 root Exp $
 */

#ifndef _LIBBRIDGE_H
#define _LIBBRIDGE_H

#include <net/if.h>
#include <linux/br.h>
#include <asm/param.h>

struct bridge;
struct port;

struct bridge
{
	struct bridge *next;

	int ifindex;
	char ifname[IFNAMSIZ];
	struct port *firstport;
	struct port *ports[256];
	struct bridge_info info;
};

struct port
{
	struct port *next;

	int index;
	int ifindex;
	struct bridge *parent;
	struct port_info info;
};

extern int br_socket_fd;
extern struct bridge *bridge_list;

/* libbridge_if.c */
struct bridge *br_find_bridge(char *brname);
void br_add_bridge(char *brname);
void br_del_bridge(char *brname);
void br_add_interface(struct bridge *br, int ifindex);
void br_del_interface(struct bridge *br, int ifindex);
void br_set_bridge_forward_delay(struct bridge *br, int forward_delay);
void br_set_bridge_hello_time(struct bridge *br, int hello_time);
void br_set_bridge_max_age(struct bridge *br, int max_age);
void br_set_ageing_time(struct bridge *br, int ageing_time);
void br_set_gc_interval(struct bridge *br, int gc_interval);
void br_set_stp_state(struct bridge *br, int stp_state);
void br_set_bridge_priority(struct bridge *br, int bridge_priority);
void br_set_port_priority(struct port *p, int port_priority);
void br_set_path_cost(struct port *p, int path_cost);
int br_get_fdb_list(struct bridge *br, struct fdb_entry *fdbs, int num);

/* libbridge_init.c */
void br_read_info(struct bridge *br);
void br_read_port_info(struct port *p);
void br_make_port_list(struct bridge *br);
void br_make_bridge_list(void);
void br_init(void);
void br_refresh(void);

/* libbridge_misc.c */
void br_dump_bridge_id(unsigned char *x);
void br_show_timer(int hz);
void br_dump_port_info(struct port *p);
void br_dump_info(struct bridge *br);

#endif
