/*
 * $Id: libbridge_init.c,v 1.2 2000/01/19 12:37:50 root Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include "libbridge.h"

int br_socket_fd;
struct bridge *bridge_list;

void br_read_info(struct bridge *br)
{
	unsigned long *i;
	struct ifreq ifr;

	if_indextoname(br->ifindex, br->ifname);
	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_GET_BRIDGE_INFO;
	i[1] = (unsigned long)&br->info;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("br_read_info");
		exit(-1);
	}
}

void br_read_port_info(struct port *p)
{
	unsigned long *i;
	struct ifreq ifr;

	memcpy(ifr.ifr_name, p->parent->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_GET_PORT_INFO;
	i[1] = (unsigned long)&p->info;
	i[2] = p->index;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("br_read_port_info");
		exit(-1);
	}
}

void br_make_port_list(struct bridge *br)
{
	unsigned long *i;
	int ifindices[256];
	struct ifreq ifr;
	int j;

	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_GET_PORT_LIST;
	i[1] = (unsigned long)ifindices;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("br_make_port_list");
		exit(-1);
	}

	for (j=255;j>=0;j--) {
		struct port *p;

		if (!ifindices[j])
			continue;

		p = malloc(sizeof(struct port));
		br->ports[j] = p;
		p->next = br->firstport;
		br->firstport = p;
		p->index = j;
		p->ifindex = ifindices[j];
		p->parent = br;

		br_read_port_info(p);
	}
}

void br_make_bridge_list()
{
	unsigned long arg[2];
	int i;
	int ifindices[32];
	int num;

	arg[0] = BRCTL_GET_BRIDGES;
	arg[1] = (unsigned long)ifindices;
	ifindices[0] = 32;
	num = ioctl(br_socket_fd, SIOCGIFBR, arg);
	if (num < 0) {
		perror("br_make_bridge_list");
		return;
	}

	bridge_list = NULL;
	for (i=0;i<num;i++) {
		struct bridge *br;

		br = malloc(sizeof(struct bridge));
		memset(br, 0, sizeof(struct bridge));
		br->ifindex = ifindices[i];
		br->firstport = NULL;
		br->next = bridge_list;
		bridge_list = br;

		br_read_info(br);
		br_make_port_list(br);
	}
}

void br_init()
{
	if ((br_socket_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		perror("opening AF_INET socket");
		exit(-1);
	}

	br_make_bridge_list();
}

void br_refresh()
{
	struct bridge *b;
	struct port *p;

	b = bridge_list;
	while (b != NULL) {
		struct bridge *bnext;

		p = b->firstport;
		while (p != NULL) {
			struct port *pnext;

			pnext = p->next;
			free(p);
			p = pnext;
		}

		bnext = b->next;
		free(b);
		b = bnext;
	}

	br_make_bridge_list();
}
