/*
 * $Id: libbridge_misc.c,v 1.4 2000/01/19 01:06:33 root Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <asm/param.h>
#include "libbridge.h"

static char *state_names[5] = {"disabled", "listening", "learning", "forwarding", "blocking"};

static char *get_state_name(int state)
{
	if (state >= 0 && state <= 4)
		return state_names[state];

	return "<INVALID STATE>";
}

void br_dump_bridge_id(unsigned char *x)
{
	printf("%.2x%.2x.%.2x%.2x%.2x%.2x%.2x%.2x", x[0], x[1], x[2], x[3],
	       x[4], x[5], x[6], x[7]);
}

void br_show_timer(int hz)
{
	printf("%4i.%.2i", hz/HZ, (100*(hz % HZ))/HZ);
}

void br_dump_port_info(struct port *p)
{
	char ifname[IFNAMSIZ];
	struct port_info *pi;

	pi = &p->info;

	printf("%s (%i)\n", if_indextoname(p->ifindex, ifname), p->index);
	printf(" port id\t\t%.2x\t\t\t", pi->port_id);
	printf("state\t\t\t%s\n", get_state_name(pi->state));
	printf(" designated root\t");
	br_dump_bridge_id((unsigned char *)&pi->designated_root);
	printf("\tdesignated port\t\t%.2x\n", pi->designated_port);
	printf(" designated bridge\t");
	br_dump_bridge_id((unsigned char *)&pi->designated_bridge);
	printf("\tmessage age\t\t");
	br_show_timer(pi->message_age_timer_value);
	printf("\n path cost\t\t%4i", pi->path_cost);
	printf("\t\t\tforward delay\t\t");
	br_show_timer(pi->forward_delay_timer_value);
	printf("\n designated cost\t%4i", pi->designated_cost);
	printf("\t\t\thold timer\t\t");
	br_show_timer(pi->hold_timer_value);
	printf("\n flags\t\t\t");
	if (pi->config_pending)
		printf("CONFIG_PENDING ");
	if (pi->top_change_ack)
		printf("TOPOLOGY_CHANGE_ACK ");
	printf("\n");
	printf("\n");
}

void br_dump_info(struct bridge *br)
{
	struct bridge_info *bri;
	struct port *p;

	bri = &br->info;

	printf("%s\n", br->ifname);
	if (!bri->stp_enabled) {
		printf("\tSTP disabled\n");
		return;
	}

	printf(" bridge_id\t\t");
	br_dump_bridge_id((unsigned char *)&bri->bridge_id);
	printf("\n designated root\t");
	br_dump_bridge_id((unsigned char *)&bri->designated_root);
	printf("\n root port\t\t%4i\t\t\t", bri->root_port);
	printf("path cost\t\t%4i\n", bri->root_path_cost);
	printf(" max age\t\t");
	br_show_timer(bri->max_age);
	printf("\t\t\tbridge max age\t\t");
	br_show_timer(bri->bridge_max_age);
	printf("\n hello time\t\t");
	br_show_timer(bri->hello_time);
	printf("\t\t\tbridge hello time\t");
	br_show_timer(bri->bridge_hello_time);
	printf("\n forward delay\t\t");
	br_show_timer(bri->forward_delay);
	printf("\t\t\tbridge forward delay\t");
	br_show_timer(bri->bridge_forward_delay);
	printf("\n topology change time\t");
	br_show_timer(bri->topology_change_time);
	printf("\t\t\thold time\t\t");
	br_show_timer(bri->hold_time);
	printf("\n ageing time\t\t");
	br_show_timer(bri->ageing_time);
	printf("\t\t\tgc interval\t\t");
	br_show_timer(bri->gc_interval);
	printf("\n hello timer\t\t");
	br_show_timer(bri->hello_timer_value);
	printf("\t\t\ttcn timer\t\t");
	br_show_timer(bri->tcn_timer_value);
	printf("\n topology change timer\t");
	br_show_timer(bri->topology_change_timer_value);
	printf("\t\t\tgc timer\t\t");
	br_show_timer(bri->gc_timer_value);
	printf("\n flags\t\t\t");
	if (bri->topology_change)
		printf("TOPOLOGY_CHANGE ");
	if (bri->topology_change_detected)
		printf("TOPOLOGY_CHANGE_DETECTED ");
	printf("\n");
	printf("\n");
	printf("\n");

	p = br->firstport;
	while (p != NULL) {
		br_dump_port_info(p);
		p = p->next;
	}
}
